using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using SysGen.RBuild.Framework;

namespace SysGen.BuildEngine.Framework
{
    public abstract class SysSetupComponentSetupFileWriter : AutoGeneratedInfFileWriter
    {
        public SysSetupComponentSetupFileWriter(RBuildProject project, string file)
            : base(project , file)
        {
        }

        protected virtual void WriteHeader()
        {
            WriteSection("Version");
            WriteLine("Signature = \"$Windows NT$\"");
            WriteLine();
        }

        protected virtual void WriteaAddRegDirective()
        {
            WriteSection("DefaultInstall");
            WriteLine("AddReg=Install.Reg");
            WriteLine();
        }

        public override void WriteFile()
        {
            WriteHeader();
            WriteaAddRegDirective();
            WriteContent();
        }

        protected abstract void WriteContent();
    }

    public class DesktopComponentSetupFileWriter : SysSetupComponentSetupFileWriter
    {
        public DesktopComponentSetupFileWriter(RBuildProject project, string file)
            : base(project, file)
        {
        }

        protected override void WriteContent()
        {
            WriteLine("[Install.Reg]");

            if (Project.Platform.Screensaver != null)
            {
                WriteLine("HKU,\"Control Panel\\Desktop\",\"1SCRNSAVE.EXE\",0x00000000,\"{0}\"",
                    Project.Platform.Screensaver.PlatformInstall.FullPath);
            }

            if (Project.Platform.Wallpaper != null)
            {
                WriteLine("HKU,\"Control Panel\\Desktop\",\"1Wallpaper\",0x00000000,\"{0}\"",
                    Project.Platform.Wallpaper.PlatformInstall.FullPath);
            }
        }
    }

    public class ShellComponentSetupFileWriter : SysSetupComponentSetupFileWriter
    {
        public ShellComponentSetupFileWriter(RBuildProject project, string file)
            : base(project, file)
        {
        }

        protected override void WriteContent()
        {
            WriteLine("[Install.Reg]");

            if (Project.Platform.Shell != null)
            {
                WriteLine("HKLM,\"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon\\\",\"Shell\",0x00020000,\"{0}\"",
                    Project.Platform.Shell.PlatformInstall.FullPath);
            }
        }
    }
}
