using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using SysGen.RBuild.Framework;

namespace SysGen.BuildEngine.Framework
{
    public class SysSetupFileWriter : AutoGeneratedInfFileWriter
    {
        public SysSetupFileWriter(RBuildProject project, string file)
            : base(project , file)
        {
        }

        public override void WriteFile()
        {
            WriteHeader();
            WriteInfDevicesSection();
            WriteInfRegistrationPhase2Section();
            WriteInfOleControlDllsSection();
            WriteInfAlwaysSection();
        }

        protected void WriteInfDevicesSection()
        {
            WriteLine("[DeviceInfsToInstall]");
            WriteLine("cdrom.inf");
            WriteLine("display.inf");
            WriteLine("hdc.inf");
            WriteLine("keyboard.inf");
            WriteLine("machine.inf");
            WriteLine("msmouse.inf");
            WriteLine("NET_NIC.inf");
            WriteLine("ports.inf");
            WriteLine("scsi.inf");
            WriteLine("usbport.inf");
            WriteLine();
        }

        protected void WriteInfRegistrationPhase2Section()
        {
            WriteLine("[RegistrationPhase2]");
            WriteLine("RegisterDlls=OleControlDlls");
            WriteLine();
        }

        protected void WriteInfOleControlDllsSection()
        {
            WriteLine("[OleControlDlls]");
            foreach (RBuildModule module in Project.Platform.Modules)
            {
                if (module.AutoRegister != null)
                {
                    WriteLine("{0},,{1},{2}",
                        "11",
                        module.TargetName, 
                        module.AutoRegister.RegistrationType);
                }
            }
            WriteLine();
        }

        protected void WriteInfAlwaysSection()
        {            
            WriteLine("[Infs.Always]");
            foreach (RBuildModule module in Project.Platform.Modules)
            {
                if (module.Setup != null)
                {
                    WriteLine("{0},{1}",
                        module.Setup.Name,
                        module.Setup.InstallSection);
                }
            }

            foreach (RBuildOutputFile file in Project.Files)
            {
                RBuildSetupFile setup = file as RBuildSetupFile;

                if (setup != null)
                {
                    WriteLine("{0},{1}",
                        setup.Name,
                        setup.InstallSection);
                }
            }
            WriteLine();
        }
    }
}
