using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using SysGen.BuildEngine;
using SysGen.RBuild.Framework;

namespace SysGen.BuildEngine.Handlers
{
    public abstract class AutoGeneratedFileHandler : IFileHandler
    {
        protected SysGenEngine m_SysGenEngine = null;
        protected RBuildFile m_OriginalFile = null;
        protected RBuildFile m_DestFile = null;

        public AutoGeneratedFileHandler(SysGenEngine engine)
        {
            m_SysGenEngine = engine;
        }

        public void Process(RBuildFile file)
        {
            m_OriginalFile = file;

            if (file.Name == FileName)
            {
                // Set the generated file to the temporary path
                m_DestFile = new RBuildFile();
                m_DestFile.Root = PathRoot.Intermediate;
                m_DestFile.Base = file.Base;
                m_DestFile.Name = file.Name;

                // Auto generate this file
                AutoGenerate();

                file = m_DestFile;
            }
        }

        protected abstract void AutoGenerate();

        protected abstract string FileName { get; }
    }

    public class HiveAutoGeneratedFileHandler : AutoGeneratedFileHandler
    {
        public HiveAutoGeneratedFileHandler(SysGenEngine engine)
            : base(engine)
        {
        }

        protected override string FileName
        {
            get { return "hivedef.inf"; }
        }

        protected override void AutoGenerate()
        {
            File.Copy(
                m_SysGenEngine.ResolveRBuildFilePath(m_OriginalFile),
                m_SysGenEngine.ResolveRBuildFilePath(m_DestFile));

            File.WriteAllText (m_SysGenEngine.ResolveRBuildFilePath(m_DestFile) ,
                "; Set default wallpaper\n" +
                "HKCU,\"Control Panel\\Desktop\",\"Wallpaper\",0000000000,\"%SystemRoot%\\Green.bmp\"" +
                "HKCU,\"Control Panel\\Desktop\",\"WallpaperStyle\",0x00000002,0x00000000\n" +
                "HKCU,\"Control Panel\\Desktop\",\"TileWallpaper\",0x00000002,0x00000000\n");
           
        }
    }

    public class AutorunFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "autorun.inf")
            {
                int i = 0;
            }
        }
    }

    public class SysSetupFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "syssetup.inf")
            {
                int i = 0;
            }
        }
    }

    public class TxtSetupFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "txtsetup.sif")
            {
                int i = 0;
            }
        }
    }

    public class UnattendFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "unattend.inf")
            {
                int i = 0;
            }
        }
    }

    public class DownloaderFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "downloader.xml")
            {
                int i = 0;
            }
        }
    }

    public class UnAttendedSetupFileHandler : IFileHandler
    {
        public void Process(RBuildFile file)
        {
            if (file.Name == "unattend.inf")
            {
                int i = 0;
            }
        }
    }
}
